use crate::error::{LogError, Result};
use chrono::Local;
use log::{LevelFilter, Log, Metadata, Record};
use std::fs::OpenOptions;
use std::io::{self, Write};
use std::path::PathBuf;
use std::sync::OnceLock;
use std::sync::{Arc, Mutex};

/// Configuration for logging setup
#[derive(Debug, Clone, PartialEq)]
pub struct LogConfig {
    pub log_file: Option<PathBuf>,
    pub log_level: LevelFilter,
    pub append_log: bool,
}

impl Default for LogConfig {
    fn default() -> Self {
        Self {
            log_file: None,
            log_level: LevelFilter::Info,
            append_log: false,
        }
    }
}

/// Custom writer that can write to either a file or stderr
#[derive(Debug)]
pub struct LogWriter {
    file: Option<std::fs::File>,
}

impl LogWriter {
    pub fn new(file: Option<std::fs::File>) -> Self {
        Self { file }
    }

    pub fn write_all(&mut self, buf: &[u8]) -> io::Result<()> {
        match &mut self.file {
            Some(file) => file.write_all(buf),
            None => io::stderr().write_all(buf),
        }
    }

    pub fn flush(&mut self) -> io::Result<()> {
        match &mut self.file {
            Some(file) => file.flush(),
            None => io::stderr().flush(),
        }
    }
}

impl Write for LogWriter {
    fn write(&mut self, buf: &[u8]) -> io::Result<usize> {
        self.write_all(buf)?;
        Ok(buf.len())
    }

    fn flush(&mut self) -> io::Result<()> {
        self.flush()
    }
}

#[derive(Debug)]
struct SimpleLogger {
    writer: Arc<Mutex<LogWriter>>,
    level: Arc<Mutex<LevelFilter>>,
}

impl SimpleLogger {
    fn new(writer: Arc<Mutex<LogWriter>>, level: Arc<Mutex<LevelFilter>>) -> Self {
        Self { writer, level }
    }

    fn get_level(&self) -> Result<LevelFilter> {
        self.level
            .lock()
            .map_err(|e| LogError::LockPoisoned(e.to_string()).into())
            .map(|guard| *guard)
    }

    fn write_log(&self, message: &str) -> Result<()> {
        self.writer
            .lock()
            .map_err(|e| LogError::LockPoisoned(e.to_string()).into())
            .and_then(|mut writer| {
                writer
                    .write_all(message.as_bytes())
                    .and_then(|_| writer.flush())
                    .map_err(|e| LogError::Io(e).into())
            })
    }
}

impl Log for SimpleLogger {
    fn enabled(&self, metadata: &Metadata) -> bool {
        self.get_level()
            .map(|level| metadata.level() <= level)
            .unwrap_or(false)
    }

    fn log(&self, record: &Record) {
        if !self.enabled(record.metadata()) {
            return;
        }

        let message = format!(
            "{} [{:<5}] - {}\n",
            Local::now().format("%Y-%m-%d %H:%M:%S%.3f"),
            record.level(),
            record.args()
        );

        if let Err(e) = self.write_log(&message) {
            eprintln!("Failed to write log message: {}", e);
        }
    }

    fn flush(&self) {
        if let Err(e) = self
            .writer
            .lock()
            .map_err(|e| LogError::LockPoisoned(e.to_string()))
            .and_then(|mut w| w.flush().map_err(LogError::Io))
        {
            eprintln!("Failed to flush logger: {}", e);
        }
    }
}

static LOGGER: OnceLock<Arc<SimpleLogger>> = OnceLock::new();
static LEVEL: OnceLock<Arc<Mutex<LevelFilter>>> = OnceLock::new();

/// Initialize logging with enhanced features
///
/// # Errors
///
/// Returns an error if:
/// - Failed to create log directory
/// - Failed to open log file
/// - Failed to write initial log header
/// - Failed to set global logger
/// - Logger is already initialized
pub fn init_logging(config: LogConfig) -> Result<()> {
    let log_file = if let Some(log_path) = &config.log_file {
        // Create parent directory if it doesn't exist
        if let Some(parent) = log_path.parent() {
            std::fs::create_dir_all(parent).map_err(LogError::Io)?;
        }

        let file = OpenOptions::new()
            .create(true)
            .write(true)
            .append(config.append_log)
            .truncate(!config.append_log)
            .open(log_path)
            .map_err(LogError::Io)?;

        // Write header for new log files
        if !config.append_log {
            writeln!(
                &file,
                "=== Log started at {} ===",
                Local::now().format("%Y-%m-%d %H:%M:%S")
            )
            .map_err(LogError::Io)?;
        }
        Some(file)
    } else {
        None
    };

    let level = LEVEL
        .get_or_init(|| Arc::new(Mutex::new(config.log_level)))
        .clone();

    {
        let mut lvl = level
            .lock()
            .map_err(|e| LogError::LockPoisoned(e.to_string()))?;
        *lvl = config.log_level;
    }

    let writer = Arc::new(Mutex::new(LogWriter::new(log_file)));
    let logger = Arc::new(SimpleLogger::new(writer.clone(), level.clone()));

    // Try to set the global logger
    if LOGGER.get().is_some() {
        return Err(LogError::AlreadyInitialized.into());
    }

    // Set the logger and store it
    log::set_boxed_logger(Box::new(SimpleLogger::new(writer, level.clone())))
        .map_err(|e| LogError::Init(e.to_string()))?;

    // Store our logger instance
    if LOGGER.set(logger).is_err() {
        return Err(LogError::AlreadyInitialized.into());
    }

    log::set_max_level(config.log_level);

    // Log initial configuration
    log::info!(
        "Logging initialized (level: {}, output: {})",
        config.log_level,
        config
            .log_file
            .as_ref()
            .map(|p| p.display().to_string())
            .unwrap_or_else(|| "console".to_string())
    );

    Ok(())
}
